Imports System.Net
Imports System.Net.Sockets

''' <summary>
''' Dieses Form kann sowohl als Server-Anwendung laufen, als auch als 
'''  Client-Anwendung.
''' </summary>
Public Class frmMain

    Private WithEvents _ServerOrClient As TCPBase 'polymorphes Objekt

    Public Sub New()
        InitializeComponent()
        With Application.OpenForms 'bei mehreren Instanzen versetzt anordnen
            If .Count > 0 Then
                Me.Location = .Item(.Count - 1).Location + New Size(30, 50)
            Else
                Me.Location = Screen.PrimaryScreen.WorkingArea.Location
            End If
        End With
    End Sub

    Private Sub MenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) _
            Handles SendMessageToolStripMenuItem.Click, _
                AddFormToolStripMenuItem.Click, _
                BeServerToolStripMenuItem.Click, _
                BeClientToolStripMenuItem.Click, _
                TestToolStripMenuItem.Click
        Select Case True
            Case sender Is SendMessageToolStripMenuItem
                If _ServerOrClient Is Nothing Then
                    DisplayStatusLine("Fehler:", Lf, "keine Verbindung")
                Else
                    _ServerOrClient.Send(Me.rtbSend.Text)
                End If

            Case sender Is AddFormToolStripMenuItem
                Call (New frmMain()).Show(Me)

            Case sender Is BeServerToolStripMenuItem
                DisposeAll(_ServerOrClient)
                Dim EP As IPEndPoint = GetEP()
                If EP Is Nothing Then Return
                _ServerOrClient = New Server(EP)
                DisplayStatusLine("Hi!", Lf, "Ich bin Server, hre auf:", _
                                  Lf, EP)
                Me.Text = "Server"

            Case sender Is BeClientToolStripMenuItem
                DisposeAll(_ServerOrClient)
                Dim EP As IPEndPoint = GetEP()
                If EP Is Nothing Then Return
                ' Sehr rgerlich: der TcpClient-Konstruktor New(EP as IPEndPoint)
                '  produziert auf Einzelplatzsystemen einen unntigen Fehler, 
                '  weil er  (undokumentiert!) TcpClient.ExclusiveAddressUse=True 
                '  voreinstellt. 
                Dim TC As New TcpClient
                TC.ExclusiveAddressUse = False
                Try
                    TC.Connect(EP)
                Catch ex As Exception
                    TC.Close()
                    DisplayStatusLine("Fehler:", Lf, ex.Message)
                    Return
                End Try
                _ServerOrClient = New Client(TC)
                DisplayStatusLine("Hi!", Lf, "Client, verbunden mit:", Lf, EP)
                Me.Text = "Client"

            Case sender Is TestToolStripMenuItem

        End Select
    End Sub

    ' Diese Sub ist normalerweise in der Designer.vb. Wir wollen jedoch
    '  sicherstellen, das _ServerOrClient *vor* den Koponenten disposed wird. 
    '  Sonst werden womglich noch Daten empfangen, und entsteht ein Fehler 
    '  beim Darstellen.
    Protected Overrides Sub Dispose(ByVal disposing As Boolean)
        DisposeAll(_ServerOrClient)
        If disposing AndAlso components IsNot Nothing Then
            components.Dispose()
        End If
        MyBase.Dispose(disposing)
    End Sub

#Region "ServerOrClient-Ereignisverarbeitung"

    Private Sub ServerOrClient_ChatMessage(ByVal Sender As Object, _
            ByVal e As MessageEventargs) Handles _ServerOrClient.ChatMessage

        DisplayLine(e.Message)
    End Sub

    Private Sub _ServerOrClient_Disposed(ByVal Sender As TCPBase) _
            Handles _ServerOrClient.Disposed
        _ServerOrClient = Nothing
        Me.Text = "Offline"
    End Sub

    Private Sub ServerOrClient_StatusMessage(ByVal Sender As Object, _
            ByVal e As MessageEventargs) Handles _ServerOrClient.StatusMessage

        DisplayStatusLine(e.Message)
    End Sub

#End Region 'ServerOrClient-Ereignisverarbeitung

    Private Function GetEP() As IPEndPoint
        Dim DefaultIP As String = ""
        Dim addresses As IPAddress() = _
            Dns.GetHostEntry(My.Computer.Name).AddressList
        If addresses.Length = 1 Then DisplayStatusLine("Sie sind Offline!")
        Array.Reverse(addresses)
        For Each addr As IPAddress In addresses
            If addr.AddressFamily = AddressFamily.InterNetwork Then
                DefaultIP = String.Concat(addr, " : 12345")
                Exit For
            End If
        Next
        Dim InputIP As String = InputBox( _
           "(voreingestellt: eigene IP und willkrlicher Port)", _
           "IP eingeben", _
           DefaultIP)
        Try
            Dim Splitted As String() = InputIP.Split(":"c)
            Dim IPAs As IPAddress() = Dns.GetHostAddresses(Splitted(0))
            Return New IPEndPoint(IPAs(0), Integer.Parse(Splitted(1)))
        Catch ex As Exception
            DisplayStatusLine("Die IP '", InputIP, _
                              "' konnte nicht aufgelst werden")
            Return Nothing
        End Try
    End Function

    Private Sub DisplayLine(ByVal Msg As String)
        Me.rtbLog.AppendText(Msg & Lf)
    End Sub

    ''' <summary>Meldung hellgelb hinterlegt anzeigen</summary>
    Private Sub DisplayStatusLine(ByVal ParamArray Items() As Object)
        Static StatusLineColor As Color = Color.FromArgb(255, 255, 200)
        Me.rtbLog.SelectionBackColor = StatusLineColor
        DisplayLine(String.Concat(Items))
        Me.rtbLog.SelectionBackColor = Color.White
    End Sub

End Class